/* 
 * $Id: x11merge.h,v 1.3 2003/02/16 12:43:37 isizaka Exp isizaka $
 * 
 * This file is part of "Ngraph for X11".
 * 
 * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
 * 
 * "Ngraph for X11" is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * "Ngraph for X11" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

/**
 *
 * $Log: x11merge.h,v $
 * Revision 1.3  2003/02/16 12:43:37  isizaka
 * for release 6.13.18
 *
 * Revision 1.2  2002/07/06 08:57:25  isizaka
 * change to GPL.
 *
 * Revision 1.1  1999/03/17 13:29:01  isizaka
 * Initial revision
 *
 *
 **/

void MergeWinUpdate(int clear);
void CmMergeMenu(Widget w,XtPointer client_data,XtPointer call_data);
void CmMergeWindow(Widget w,XtPointer client_data,XtPointer call_data);
void MergeWindowUnmap(Widget w,XtPointer client_data,XtPointer call_data);
