/* 
 * $Id: motif12.h,v 1.6 2003/02/16 12:43:37 isizaka Exp isizaka $
 * 
 * This file is part of "Ngraph for X11".
 * 
 * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
 * 
 * "Ngraph for X11" is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * "Ngraph for X11" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

/**
 *
 * $Log: motif12.h,v $
 * Revision 1.6  2003/02/16 12:43:37  isizaka
 * for release 6.13.18
 *
 * Revision 1.5  2002/07/06 08:57:25  isizaka
 * change to GPL.
 *
 * Revision 1.4  2001/03/23 12:17:43  isizaka
 * for 6.3.13
 *
 * Revision 1.3  1999/04/15 12:14:26  isizaka
 * for release 6.03.01
 *
 * Revision 1.2  1999/04/11 06:09:20  isizaka
 * *** empty log message ***
 *
 * Revision 1.1  1999/04/06 10:33:00  isizaka
 * Initial revision
 *
 *
 **/

#ifdef MOTIF12

#include "ComboBox.h"

#define XmSET 1
#define XmUNSET 0
#define XmNEAR_SLIDER 1
#define XmNEAR_BORDER 1
#define XmINDICATOR_NONE 0

#ifdef CDE

#define ZEROBASECOMBOBOX
#define XmNcomboBoxType DtNcomboBoxType
#define XmDROP_DOWN_LIST DtDROP_DOWN_LIST
#define XmDROP_DOWN_COMBO_BOX DtDROP_DOWN_COMBO_BOX
#define XmComboBoxCallbackStruct DtComboBoxCallbackStruct
#define XmNselectionCallback DtNselectionCallback
#define XmCreateComboBox DtCreateComboBox
#define COMBOTEXT "*Text"
#define COMBOLIST "*List"

#else  /* else CDE */

#define ZEROBASECOMBOBOX
#define XmNcomboBoxType XmNtype
#define XmDROP_DOWN_LIST XmDROP_DOWN_LIST_BOX
#define XmComboBoxCallbackStruct DtComboBoxCallbackStruct
#define XmCreateComboBox DtCreateComboBox
#define COMBOTEXT "*Text"
#define COMBOLIST "*List"

#endif /* end of CDE */

#else /* else MOTIF12 */

#define COMBOTEXT "*Text"
#define COMBOLIST "*List"

#endif /* end of MOTIF12 */
